/* mimeview.c */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkctree.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkvpaned.h>
#include <gtk/gtktext.h>
#include <gtk/gtksignal.h>
#include <stdio.h>

#include "intl.h"
#include "main.h"
#include "mimeview.h"
#include "procmime.h"
#include "utils.h"

typedef enum
{
	COL_MIMETYPE = 0,
	COL_SIZE     = 1,
	COL_NAME     = 2
} MimeViewColumnPos;

#define N_MIMEVIEW_COLS	3

static void mimeview_show_message_part(MimeView		*mimeview,
				       MimeInfo		*partinfo);
static void mimeview_selected	      (GtkCTree		*ctree,
				       GtkCTreeNode	*node,
				       gint		 column,
				       MimeView		*mimeview);

MimeView *mimeview_create(void)
{
	MimeView *mimeview;
	GtkWidget *paned;
	GtkWidget *scrolledwin;
	GtkWidget *ctree;
	GtkWidget *vbox;
	gchar *titles[] = {_("MIME Type"), _("Size"), _("Name")};
	gint i;

	debug_print(_("Creating MIME view...\n"));
	mimeview = g_new0(MimeView, 1);

	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_ALWAYS);
	gtk_widget_set_usize(scrolledwin, -1, 80);

	ctree = gtk_ctree_new_with_titles(N_MIMEVIEW_COLS, 0, titles);
	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
	gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_NONE);
	gtk_clist_set_column_justification(GTK_CLIST(ctree), COL_SIZE,
					   GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_width(GTK_CLIST(ctree), COL_MIMETYPE, 180);
	gtk_clist_set_column_width(GTK_CLIST(ctree), COL_SIZE, 40);
	for (i = 0; i < N_MIMEVIEW_COLS; i++)
		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[i].button,
				       GTK_CAN_FOCUS);
	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);

	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
			   GTK_SIGNAL_FUNC(mimeview_selected), mimeview);

	vbox = gtk_vbox_new(FALSE, 0);

	paned = gtk_vpaned_new();
	gtk_paned_add1(GTK_PANED(paned), scrolledwin);
	gtk_paned_add2(GTK_PANED(paned), vbox);
	gtk_widget_show_all(paned);

	mimeview->paned       = paned;
	mimeview->scrolledwin = scrolledwin;
	mimeview->ctree       = ctree;
	mimeview->vbox        = vbox;

	return mimeview;
}

void mimeview_init(MimeView *mimeview)
{
}

void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
			   const gchar *file)
{
	GtkCList *clist = GTK_CLIST(mimeview->ctree);
	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
	GtkText *text = GTK_TEXT(mimeview->textview->text);
	GtkCTreeNode *node;
	gchar buf[BUFFSIZE];
	FILE *fp;
	gchar *boundary;
	gint boundary_len;
	gchar *p;
	glong fpos, prev_fpos;

	gtk_clist_clear(clist);
	gtk_text_freeze(text);
	gtk_text_backward_delete(text, gtk_text_get_length(text));
	gtk_text_thaw(text);
	g_free(mimeview->boundary);
	mimeview->boundary = NULL;
	g_free(mimeview->file);
	mimeview->file = NULL;

	if (!mimeinfo) return;
	if (mimeinfo->boundary) {
		mimeview->boundary = boundary =
			g_strconcat("--", mimeinfo->boundary, NULL);
		boundary_len = strlen(boundary);
	} else
		return;

	g_return_if_fail(file != NULL);

	if ((fp = fopen(file, "r")) == NULL) {
		fprintf(stderr, "%s: ", file);
		perror("fopen");
		return;
	}
	mimeview->file = g_strdup(file);

	/* skip headers */
	while (fgets(buf, sizeof(buf), fp) != NULL)
		if (buf[0] == '\r' || buf[0] == '\n') break;

	/* look for first boundary */
	while ((p = fgets(buf, sizeof(buf), fp)) != NULL)
		if (!strncmp(buf, boundary, boundary_len)) break;
	if ((fpos = ftell(fp)) < 0)
		perror("ftell");

	for (;;) {
		MimeInfo *partinfo;
		gchar *str[N_MIMEVIEW_COLS];
		gint size;

		if (!p) break;

		prev_fpos = fpos;

		partinfo = procmime_scan_mime_header(fp);
		if (!partinfo) break;
		partinfo->fpos = fpos;

		/* look for next boundary */
		buf[0] = '\0';
		while ((p = fgets(buf, sizeof(buf), fp)) != NULL)
			if (!strncmp(buf, boundary, boundary_len)) break;
		if ((fpos = ftell(fp)) < 0)
			perror("ftell");

		size = fpos - prev_fpos - strlen(buf);

		str[COL_MIMETYPE] =
			partinfo->content_type ? partinfo->content_type : "";

		if (size >= 0)
			str[COL_SIZE] = to_human_readable(size);
		else
			str[COL_SIZE] = "";

		str[COL_NAME] = partinfo->name ? partinfo->name : "";

		node = gtk_ctree_insert_node(ctree, NULL, NULL, str, 0,
					     NULL, NULL, NULL, NULL,
					     TRUE, FALSE);
		gtk_ctree_node_set_row_data(ctree, node, partinfo);
	}

	fclose(fp);

	if ((node = gtk_ctree_node_nth(ctree, 0)))
		gtk_ctree_select(ctree, node);
}

static void mimeview_show_message_part(MimeView *mimeview, MimeInfo *partinfo)
{
	FILE *fp;

	if (!partinfo) return;
	if (!mimeview->file) return;
	if (!mimeview->boundary) return;

	if ((fp = fopen(mimeview->file, "r")) == NULL) {
		fprintf(stderr, "%s: ", mimeview->file);
		perror("fopen");
		return;
	}

	if (fseek(fp, partinfo->fpos, SEEK_SET) < 0) {
		perror("fseek");
		fclose(fp);
		return;
	}

	textview_show_part(mimeview->textview, partinfo, fp,
			   mimeview->boundary);

	fclose(fp);
}

static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
			      MimeView *mimeview)
{
	MimeInfo *partinfo;

	partinfo = gtk_ctree_node_get_row_data(ctree, node);
	mimeview_show_message_part(mimeview, partinfo);
}
